lines_default = '''
<div class="yap module">
  <h1 class="yap module-title" id="tests-mock-file">
    <a aria-hidden="true" href="#tests-mock-file" tab_index="-1">
      <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
        <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
      </svg>
    </a>tests.mock_file
  </h1>
  <div class="yap doc-str-content">module docstring content more content</div>
  <section class="yap func">
    <h2 class="yap func-title" id="mock-function">
      <a aria-hidden="true" href="#mock-function" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>mock_function
    </h2>
    <div class="yap func-sig-content">
      <div class="yap func-sig">
        <span>mock_function(</span>
        <div class="yap func-sig-params">
          <div class="yap func-sig-param">param_a, </div>
          <div class="yap func-sig-param">param_b=2</div>
        </div>
        <span>)</span>
      </div>
    </div>
    <div class="yap doc-str-content">
      <Markdown class="doc-str-content">A mock function returning a sum of param_a and param_b if positive numbers, else None</Markdown>
      <h3 class="yap doc-str-heading">Parameters</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_a</div>
          <div class="yap doc-str-elem-type">int</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">A *test* _param_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_b</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Another *test* _param_.

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Returns</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">summed_number</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">The sum of _param_a_ and _param_b_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">None</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">None returned if values are negative.</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Raises</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">ValueError</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Raises value error if params are not numbers.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-meta">
        <h3 class="yap doc-str-heading">Notes</h3>
        <Markdown class="doc-str-content">```python
print(mock_function(1, 2))
# prints 3
```

Random text

_Random table_

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
      </div>
    </div>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="parentclass">
      <a aria-hidden="true" href="#parentclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ParentClass
    </h2>
    <Markdown class="doc-str-content">A parent class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">parent_prop</div>
        <div class="yap class-prop-def-type">str</div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="parentclass">
        <a aria-hidden="true" href="#parentclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ParentClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ParentClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Parent initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="childclass">
      <a aria-hidden="true" href="#childclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ChildClass
    </h2>
    <p class="doc-str-content">Inherits from
      <a class="yap class-base" href="#parentclass">ParentClass</a>.
    </p>
    <Markdown class="doc-str-content">A child class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">param_e</div>
        <div class="yap class-prop-def-type"></div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass">
        <a aria-hidden="true" href="#childclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">param_c=1.1, </div>
            <div class="yap func-sig-param">param_d=0.9, </div>
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Child initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_c</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Yet another test param.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_d</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">And another.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass-hello">
        <a aria-hidden="true" href="#childclass-hello" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass.hello
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass.hello(</span>
          <div class="yap func-sig-params"></div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">A random class method returning &quot;hello&quot;</Markdown>
        <h3 class="yap doc-str-heading">Returns</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">str</div>
            <div class="yap doc-str-elem-type">saying_hello</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">A string saying &quot;hello&quot;</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
</div>
'''

lines_custom = '''
---
import { Markdown } from 'astro/components';
import PageLayout from '../layouts/PageLayout.astro'
---

<PageLayout>
<div class="yap module">
  <h1 class="yap module-title" id="tests-mock-file">
    <a aria-hidden="true" href="#tests-mock-file" tab_index="-1">
      <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
        <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
      </svg>
    </a>tests.mock_file
  </h1>
  <div class="yap doc-str-content">module docstring content more content</div>
  <section class="yap func">
    <h2 class="yap func-title" id="mock-function">
      <a aria-hidden="true" href="#mock-function" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>mock_function
    </h2>
    <div class="yap func-sig-content">
      <div class="yap func-sig">
        <span>mock_function(</span>
        <div class="yap func-sig-params">
          <div class="yap func-sig-param">param_a, </div>
          <div class="yap func-sig-param">param_b=2</div>
        </div>
        <span>)</span>
      </div>
    </div>
    <div class="yap doc-str-content">
      <Markdown class="doc-str-content">A mock function returning a sum of param_a and param_b if positive numbers, else None</Markdown>
      <h3 class="yap doc-str-heading">Parameters</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_a</div>
          <div class="yap doc-str-elem-type">int</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">A *test* _param_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_b</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Another *test* _param_.

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Returns</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">summed_number</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">The sum of _param_a_ and _param_b_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">None</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">None returned if values are negative.</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Raises</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">ValueError</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Raises value error if params are not numbers.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-meta">
        <h3 class="yap doc-str-heading">Notes</h3>
        <Markdown class="doc-str-content">```python
print(mock_function(1, 2))
# prints 3
```

Random text

_Random table_

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
      </div>
    </div>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="parentclass">
      <a aria-hidden="true" href="#parentclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ParentClass
    </h2>
    <Markdown class="doc-str-content">A parent class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">parent_prop</div>
        <div class="yap class-prop-def-type">str</div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="parentclass">
        <a aria-hidden="true" href="#parentclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ParentClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ParentClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Parent initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="childclass">
      <a aria-hidden="true" href="#childclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ChildClass
    </h2>
    <p class="doc-str-content">Inherits from
      <a class="yap class-base" href="#parentclass">ParentClass</a>.
    </p>
    <Markdown class="doc-str-content">A child class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">param_e</div>
        <div class="yap class-prop-def-type"></div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass">
        <a aria-hidden="true" href="#childclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">param_c=1.1, </div>
            <div class="yap func-sig-param">param_d=0.9, </div>
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Child initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_c</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Yet another test param.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_d</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">And another.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass-hello">
        <a aria-hidden="true" href="#childclass-hello" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass.hello
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass.hello(</span>
          <div class="yap func-sig-params"></div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">A random class method returning &quot;hello&quot;</Markdown>
        <h3 class="yap doc-str-heading">Returns</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">str</div>
            <div class="yap doc-str-elem-type">saying_hello</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">A string saying &quot;hello&quot;</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
</div>
</PageLayout>
'''

astro_file_default = '''
<div class="yap module">
  <h1 class="yap module-title" id="test-mock-file">
    <a aria-hidden="true" href="#test-mock-file" tab_index="-1">
      <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
        <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
      </svg>
    </a>test.mock_file
  </h1>
  <div class="yap doc-str-content">module docstring content more content</div>
  <section class="yap func">
    <h2 class="yap func-title" id="mock-function">
      <a aria-hidden="true" href="#mock-function" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>mock_function
    </h2>
    <div class="yap func-sig-content">
      <div class="yap func-sig">
        <span>mock_function(</span>
        <div class="yap func-sig-params">
          <div class="yap func-sig-param">param_a, </div>
          <div class="yap func-sig-param">param_b=2</div>
        </div>
        <span>)</span>
      </div>
    </div>
    <div class="yap doc-str-content">
      <Markdown class="doc-str-content">A mock function returning a sum of param_a and param_b if positive numbers, else None</Markdown>
      <h3 class="yap doc-str-heading">Parameters</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_a</div>
          <div class="yap doc-str-elem-type">int</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">A *test* _param_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_b</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Another *test* _param_.

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Returns</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">summed_number</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">The sum of _param_a_ and _param_b_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">None</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">None returned if values are negative.</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Raises</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">ValueError</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Raises value error if params are not numbers.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-meta">
        <h3 class="yap doc-str-heading">Notes</h3>
        <Markdown class="doc-str-content">```python
print(mock_function(1, 2))
# prints 3
```

Random text

_Random table_

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
      </div>
    </div>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="parentclass">
      <a aria-hidden="true" href="#parentclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ParentClass
    </h2>
    <Markdown class="doc-str-content">A parent class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">parent_prop</div>
        <div class="yap class-prop-def-type">str</div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="parentclass">
        <a aria-hidden="true" href="#parentclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ParentClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ParentClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Parent initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="childclass">
      <a aria-hidden="true" href="#childclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ChildClass
    </h2>
    <p class="doc-str-content">Inherits from
      <a class="yap class-base" href="#parentclass">ParentClass</a>.
    </p>
    <Markdown class="doc-str-content">A child class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">param_e</div>
        <div class="yap class-prop-def-type"></div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass">
        <a aria-hidden="true" href="#childclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">param_c=1.1, </div>
            <div class="yap func-sig-param">param_d=0.9, </div>
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Child initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_c</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Yet another test param.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_d</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">And another.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass-hello">
        <a aria-hidden="true" href="#childclass-hello" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass.hello
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass.hello(</span>
          <div class="yap func-sig-params"></div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">A random class method returning &quot;hello&quot;</Markdown>
        <h3 class="yap doc-str-heading">Returns</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">str</div>
            <div class="yap doc-str-elem-type">saying_hello</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">A string saying &quot;hello&quot;</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
</div>
'''

astro_file_custom = '''
---
import { Markdown } from 'astro/components';
import PageLayout from '../layouts/PageLayout.astro'
---

<PageLayout>
<div class="yap module">
  <h1 class="yap module-title" id="test-mock-file">
    <a aria-hidden="true" href="#test-mock-file" tab_index="-1">
      <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
        <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
      </svg>
    </a>test.mock_file
  </h1>
  <div class="yap doc-str-content">module docstring content more content</div>
  <section class="yap func">
    <h2 class="yap func-title" id="mock-function">
      <a aria-hidden="true" href="#mock-function" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>mock_function
    </h2>
    <div class="yap func-sig-content">
      <div class="yap func-sig">
        <span>mock_function(</span>
        <div class="yap func-sig-params">
          <div class="yap func-sig-param">param_a, </div>
          <div class="yap func-sig-param">param_b=2</div>
        </div>
        <span>)</span>
      </div>
    </div>
    <div class="yap doc-str-content">
      <Markdown class="doc-str-content">A mock function returning a sum of param_a and param_b if positive numbers, else None</Markdown>
      <h3 class="yap doc-str-heading">Parameters</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_a</div>
          <div class="yap doc-str-elem-type">int</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">A *test* _param_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">param_b</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Another *test* _param_.

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Returns</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name">summed_number</div>
          <div class="yap doc-str-elem-type">int | float</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">The sum of _param_a_ and _param_b_.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">None</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">None returned if values are negative.</Markdown>
        </div>
      </div>
      <h3 class="yap doc-str-heading">Raises</h3>
      <div class="yap doc-str-elem-container">
        <div class="yap doc-str-elem-def">
          <div class="yap doc-str-elem-name"></div>
          <div class="yap doc-str-elem-type">ValueError</div>
        </div>
        <div class="yap doc-str-elem-desc">
          <Markdown class="doc-str-content">Raises value error if params are not numbers.</Markdown>
        </div>
      </div>
      <div class="yap doc-str-meta">
        <h3 class="yap doc-str-heading">Notes</h3>
        <Markdown class="doc-str-content">```python
print(mock_function(1, 2))
# prints 3
```

Random text

_Random table_

| col A |: col B |
|-------|--------|
| boo   | baa    |</Markdown>
      </div>
    </div>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="parentclass">
      <a aria-hidden="true" href="#parentclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ParentClass
    </h2>
    <Markdown class="doc-str-content">A parent class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">parent_prop</div>
        <div class="yap class-prop-def-type">str</div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="parentclass">
        <a aria-hidden="true" href="#parentclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ParentClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ParentClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Parent initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
  <section class="yap class">
    <h2 class="yap class-title" id="childclass">
      <a aria-hidden="true" href="#childclass" tab_index="-1">
        <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
          <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
        </svg>
      </a>ChildClass
    </h2>
    <p class="doc-str-content">Inherits from
      <a class="yap class-base" href="#parentclass">ParentClass</a>.
    </p>
    <Markdown class="doc-str-content">A child class</Markdown>
    <h3 class="yap doc-str-heading">Properties</h3>
    <div class="yap class-prop-elem-container">
      <div class="yap class-prop-def">
        <div class="yap class-prop-def-name">param_e</div>
        <div class="yap class-prop-def-type"></div>
      </div>
      <div class="yap class-prop-def-desc"></div>
    </div>
    <h3 class="yap doc-str-heading">Methods</h3>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass">
        <a aria-hidden="true" href="#childclass" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass(</span>
          <div class="yap func-sig-params">
            <div class="yap func-sig-param">param_c=1.1, </div>
            <div class="yap func-sig-param">param_d=0.9, </div>
            <div class="yap func-sig-param">**kwargs</div>
          </div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">Child initialisation.</Markdown>
        <h3 class="yap doc-str-heading">Parameters</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_c</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Yet another test param.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">param_d</div>
            <div class="yap doc-str-elem-type">float</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">And another.</Markdown>
          </div>
        </div>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">**kwargs</div>
            <div class="yap doc-str-elem-type"></div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">Keyword args.</Markdown>
          </div>
        </div>
      </div>
    </section>
    <section class="yap func">
      <h2 class="yap func-title" id="childclass-hello">
        <a aria-hidden="true" href="#childclass-hello" tab_index="-1">
          <svg ariaHidden="true" class="heading-icon" height="15px" viewbox="0 0 20 20" width="15px" xmlns="http://www.w3.org/2000/svg">
            <path clip-rule="evenodd" d="
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
" fill-rule="evenodd"></path>
          </svg>
        </a>ChildClass.hello
      </h2>
      <div class="yap func-sig-content">
        <div class="yap func-sig">
          <span>ChildClass.hello(</span>
          <div class="yap func-sig-params"></div>
          <span>)</span>
        </div>
      </div>
      <div class="yap doc-str-content">
        <Markdown class="doc-str-content">A random class method returning &quot;hello&quot;</Markdown>
        <h3 class="yap doc-str-heading">Returns</h3>
        <div class="yap doc-str-elem-container">
          <div class="yap doc-str-elem-def">
            <div class="yap doc-str-elem-name">str</div>
            <div class="yap doc-str-elem-type">saying_hello</div>
          </div>
          <div class="yap doc-str-elem-desc">
            <Markdown class="doc-str-content">A string saying &quot;hello&quot;</Markdown>
          </div>
        </div>
      </div>
    </section>
  </section>
</div>
</PageLayout>
'''


"""
Uses AST and docstring_parser to parse docstrings to .astro files.

Intended for use with the Astro static site generator where further linting / linking / styling is done downstream.
"""
from __future__ import annotations

import ast
import logging

import docstring_parser
from dominate import tags, util, dom_tag, svg
from slugify import slugify

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


# custom class for markdown
class Markdown(dom_tag.dom_tag):
    pass

link_icon = f'''
M12.586 4.586a2 2 0 112.828 2.828l-3 3a2 2 0 01-2.828 0 1 1 0 00-1.414 1.414 4 4 0 005.656 0l3-3a4 4 0 00-5.656-5.656l-1.5 1.5a1 1 0 101.414 1.414l1.5-1.5zm-5 5a2 2 0 012.828 0 1 1 0 101.414-1.414 4 4 0 00-5.656 0l-3 3a4 4 0 105.656 5.656l1.5-1.5a1 1 0 10-1.414-1.414l-1.5 1.5a2 2 0 11-2.828-2.828l3-3z
'''


def heading_linker(heading_level: str,
                   heading_name: str,
                   heading_cls: str):
    """ """
    if heading_level == 'h1':
        h = tags.h1(id=slugify(heading_name),
                    cls=heading_cls)
    elif heading_level == 'h2':
        h = tags.h2(id=slugify(heading_name),
                    cls=heading_cls)
    else:
        raise NotImplementedError(f'Heading level {heading_level} is not implemented for linking.')
    with h:
        a = tags.a(aria_hidden='true',
                   tab_index='-1',
                   href=f'#{slugify(heading_name)}')
        with a:
            s = svg.svg(xmlns='http://www.w3.org/2000/svg',
                        viewbox='0 0 20 20',
                        ariaHidden='true',
                        width='15px',
                        height='15px',
                        cls='heading-icon')
            with s:
                svg.path(d=link_icon,
                         fill_rule='evenodd',
                         clip_rule='evenodd')
        util.text(heading_name)

    return h


def addMarkdown(content: str):
    """ """
    content = content.strip().strip('\n')
    md = Markdown(content,
                  cls='doc-str-content')
    return md


def process_class(ast_class: ast.ClassDef):
    """ """
    if ast_class is None:
        return
    # build class fragment
    class_fragment = tags.section(cls='yap class')
    class_fragment += heading_linker(heading_level='h2',
                                     heading_name=ast_class.name,
                                     heading_cls='yap class-title')
    # base classes
    for base in ast_class.bases:
        with class_fragment:
            base_item = tags.p(cls='doc-str-content')
            with base_item:
                util.text('Inherits from')
                tags.a(base.id,
                       cls='yap class-base',
                       href=f'#{slugify(base.id)}')
                util.text('.')
    # when the class is passed-in directly its name is captured in the member_name
    methods = []
    props = []
    for item in ast_class.body:
        if isinstance(item, ast.Expr):
            if isinstance(item.value, ast.Constant):
                desc = item.value.value.strip().lstrip('\n').rstrip('\n')
                class_fragment += addMarkdown(desc)
            else:
                raise NotImplementedError
        elif isinstance(item, ast.AnnAssign):
            props.append(item)
        elif isinstance(item, ast.FunctionDef):
            is_property = False
            for dec in item.decorator_list:
                if isinstance(dec, ast.Name):
                    if dec.id == 'property':
                        is_property = True
                    else:
                        raise NotImplementedError
                else:
                    raise NotImplementedError
            if is_property:
                props.append(item)
            else:
                methods.append(item)
    # process props
    if len(props):
        class_fragment = add_heading(doc_str_frag=class_fragment,
                                     heading='Properties')
    for prop in props:
        if hasattr(prop, 'name'):
            prop_name = prop.name
        elif hasattr(prop, 'target'):
            prop_name = prop.target.id
        else:
            raise NotImplementedError('Unable to extract property name.')
        prop_type = ''
        if hasattr(prop, 'annotation'):
            prop_type = prop.annotation.id
        prop_desc = ''
        with class_fragment:
            tags.div(
                tags.div(
                    tags.div(prop_name,
                             cls='yap class-prop-def-name'),
                    tags.div(prop_type,
                             cls='yap class-prop-def-type'),
                    cls='yap class-prop-def'),
                tags.div(prop_desc,
                         cls='yap class-prop-def-desc'),
                cls='yap class-prop-elem-container')
    # process methods
    if len(methods):
        class_fragment = add_heading(doc_str_frag=class_fragment,
                                     heading='Methods')
    for method in methods:
        with class_fragment:
            process_function(ast_function=method,
                             class_name=ast_class.name)

    return class_fragment


def process_signature(func_name: str,
                      param_names: list[str],
                      param_defaults: list[str]):
    """ """
    # process signature
    sig_fragment = tags.div(cls='yap func-sig')
    with sig_fragment:
        tags.span(f'{func_name}(')
    # nest sig params for CSS alignment
    sig_params_fragment = tags.div(cls='yap func-sig-params')
    for idx, (param_name, param_default) in enumerate(zip(param_names, param_defaults)):
        param = f'{param_name}'
        if param_default is not None:
            param += f'={param_default}'
        if idx < len(param_names) - 1:
            param += ', '
        with sig_params_fragment:
            tags.div(param,
                     cls='yap func-sig-param')
    sig_fragment += sig_params_fragment
    # close the signature
    with sig_fragment:
        tags.span(')')
    return sig_fragment


def add_heading(doc_str_frag: tags.div,
                heading: str):
    """ """
    with doc_str_frag:
        tags.h3(heading,
                cls='yap doc-str-heading')
    return doc_str_frag


def add_param_set(doc_str_frag: tags.div,
                  param_name: str,
                  param_type: list[str],
                  param_description: str):
    """ """
    if param_name is None:
        param_name = ''
    if param_type is None:
        param_types_str = 'None'
    else:
        param_types_str = ' | '.join(param_type)
    with doc_str_frag:
        tags.div(
            tags.div(
                tags.div(param_name,
                         cls='yap doc-str-elem-name'),
                tags.div(param_types_str,
                         cls='yap doc-str-elem-type'),
                cls='yap doc-str-elem-def'
            ),
            tags.div(
                addMarkdown(param_description),
                cls='yap doc-str-elem-desc'
            ),
            cls='yap doc-str-elem-container')
    return doc_str_frag


def process_docstring(doc_str: str | None,
                      param_names: list[str],
                      param_types: list[list[str]],
                      return_types: list[str]):
    """ """
    doc_str_frag = tags.div(cls='yap doc-str-content')
    if doc_str is not None:
        parsed_doc_str = docstring_parser.parse(doc_str)
        if parsed_doc_str.short_description is not None:
            doc_str_frag += addMarkdown(parsed_doc_str.short_description)
        if parsed_doc_str.long_description is not None:
            doc_str_frag += addMarkdown(parsed_doc_str.long_description)
        if (len(param_names) and parsed_doc_str.params is None) or (len(param_names) != len(parsed_doc_str.params)):
            raise ValueError('Number of docstring params does not match number of signature params')
        if parsed_doc_str.params is not None and len(parsed_doc_str.params):
            doc_str_frag = add_heading(doc_str_frag=doc_str_frag,
                                       heading='Parameters')
            for idx, param in enumerate(parsed_doc_str.params):
                param_name = param.arg_name
                if 'kwargs' in param_name:
                    param_name = param_name.lstrip('**')
                    param_name = f'**{param_name}'
                try:
                    param_idx = param_names.index(param_name)
                except ValueError:
                    raise ValueError(f'docstring param: {param_name} not found in function signature parameters.')
                param_type = param_types[param_idx]
                if param.type_name is not None and param.type_name != param_type:
                    raise ValueError(f'docstring param {param_name} type mismatch against function signature.')
                doc_str_frag = add_param_set(doc_str_frag=doc_str_frag,
                                             param_name=param_name,
                                             param_type=param_type,
                                             param_description=param.description)
        doc_str_return_types = []
        if parsed_doc_str.many_returns is not None and len(parsed_doc_str.many_returns):
            doc_str_frag = add_heading(doc_str_frag=doc_str_frag,
                                       heading='Returns')
            for doc_str_return in parsed_doc_str.many_returns:
                if doc_str_return.type_name in [None, 'None']:
                    types = None
                else:
                    types = doc_str_return.type_name.split('|')
                    types = [rt.strip() for rt in types]
                    doc_str_return_types += types
                # if there is a single return and if the return types are no specified,
                # then infer return types from the signature if available
                if len(parsed_doc_str.many_returns) == 1 \
                        and not len(doc_str_return_types):
                    types = return_types
                doc_str_frag = add_param_set(doc_str_frag=doc_str_frag,
                                             param_name=doc_str_return.return_name,
                                             param_type=types,
                                             param_description=doc_str_return.description)
        # if types were provided in both the signature and the docstring, check that these match
        if len(doc_str_return_types) and len(return_types) and not len(doc_str_return_types) == len(return_types):
            raise ValueError('Mismatching number of return types in docstring vs. function signature.')
        if len(parsed_doc_str.raises):
            doc_str_frag = add_heading(doc_str_frag=doc_str_frag,
                                       heading='Raises')
            for raises in parsed_doc_str.raises:
                doc_str_frag = add_param_set(doc_str_frag=doc_str_frag,
                                             param_name='',
                                             param_type=[raises.type_name],
                                             param_description=raises.description)
        if parsed_doc_str.deprecation is not None:
            raise NotImplementedError('Deprecation not implemented.')
        metas = []
        for met in parsed_doc_str.meta:
            if not isinstance(met, (docstring_parser.common.DocstringParam,
                                    docstring_parser.common.DocstringDeprecated,
                                    docstring_parser.common.DocstringRaises,
                                    docstring_parser.common.DocstringReturns)):
                metas.append(met)
        if len(metas):
            metas_frag = tags.div(cls='yap doc-str-meta')
            metas_frag = add_heading(doc_str_frag=metas_frag,
                                     heading='Notes')
            for meta in metas:
                metas_frag += addMarkdown(meta.description)
            doc_str_frag += metas_frag

    return doc_str_frag


def extract_types(ast_types: list[str],
                  ast_return: ast.BinOp | ast.Name):
    """ """
    if isinstance(ast_return, ast.Name):
        ast_types.append(ast_return.id)
    if hasattr(ast_return, 'left'):
        if isinstance(ast_return.left, ast.Name):
            ast_types.append(ast_return.left.id)
        elif isinstance(ast_return.left, ast.BinOp):
            extract_types(ast_types=ast_types,
                          ast_return=ast_return.left)
    if hasattr(ast_return, 'right'):
        if isinstance(ast_return.right, ast.Name):
            ast_types.append(ast_return.right.id)
        elif isinstance(ast_return.right, ast.BinOp):
            extract_types(ast_types=ast_types,
                          ast_return=ast_return.right)


def process_function(ast_function: ast.FunctionDef,
                     class_name: str = None):
    """ """
    # don't process private members
    if ast_function.name.startswith('_') and not ast_function.name == '__init__':
        return
    func_fragment = tags.section(cls='yap func')
    if class_name and ast_function.name == '__init__':
        func_name = f'{class_name}'
    elif class_name:
        func_name = f'{class_name}.{ast_function.name}'
    else:
        func_name = ast_function.name
    func_fragment += heading_linker(heading_level='h2',
                                    heading_name=func_name,
                                    heading_cls='yap func-title')
    # extract parameters, types, defaults
    param_names = []
    param_types = []
    param_defaults = []
    # pad defaults to keep in sync
    pad = len(ast_function.args.args) - len(ast_function.args.defaults)
    for idx, arg in enumerate(ast_function.args.args):
        if arg.arg == 'self':
            continue
        param_names.append(arg.arg)
        types = []
        if isinstance(arg.annotation, (ast.Name, ast.BinOp)):
            extract_types(types, arg.annotation)
        param_types.append(types)
        if idx < pad:
            param_defaults.append(None)
        else:
            param_defaults.append(getattr(ast_function.args.defaults[idx - pad], 'value'))
    if hasattr(ast_function.args, 'kwarg') and ast_function.args.kwarg is not None:
        param_names.append('**kwargs')
        param_types.append([])
        param_defaults.append(None)
    # extract return types
    return_types = []
    if isinstance(ast_function.returns, (ast.BinOp, ast.Name)):
        extract_types(ast_types=return_types,
                      ast_return=ast_function.returns)
    # process signature
    with func_fragment:
        tags.div(cls='yap func-sig-content').appendChild(
            process_signature(func_name=func_name,
                              param_names=param_names,
                              param_defaults=param_defaults))
    # process docstring
    doc_str = ast.get_docstring(ast_function)
    func_fragment.appendChild(
        process_docstring(doc_str=doc_str,
                          param_names=param_names,
                          param_types=param_types,
                          return_types=return_types))

    return func_fragment


def parse(module_name: str,
          ast_module: ast.Module,
          yap_config: dict):
    """ """
    # start the DOM fragment
    dom_fragment = tags.div(cls='yap module')
    dom_fragment += heading_linker(heading_level='h1',
                                   heading_name=module_name,
                                   heading_cls='yap module-title')
    # module docstring
    module_docstring = ast.get_docstring(ast_module)
    if module_docstring is not None:
        with dom_fragment:
            tags.div(module_docstring.replace('\n', ' ').strip(),
                     cls='yap doc-str-content')
    # iterate the module's members
    for item in ast_module.body:
        # process functions
        if isinstance(item, (ast.FunctionDef, ast.AsyncFunctionDef)):
            dom_fragment += process_function(item)
        # process classes and nested methods
        elif isinstance(item, ast.ClassDef):
            dom_fragment += process_class(item)
    astro = ''
    if yap_config['intro_template'] is not None:
        for line in yap_config['intro_template'].split('\n'):
            astro += f'{line.strip()}\n'
    astro += dom_fragment.render().strip()
    if yap_config['outro_template'] is not None:
        for line in yap_config['outro_template'].split('\n'):
            astro += f'{line.strip()}\n'

    return astro



"""
module docstring content
more content
"""
from __future__ import annotations


def mock_function(param_a: int,
                  param_b: int | float = 2) -> int | float | None:
    """
    A mock function returning a sum of param_a and param_b if positive numbers, else None

    Parameters
    ----------
    param_a
        A *test* _param_.
    param_b
        Another *test* _param_.

        | col A |: col B |
        |-------|--------|
        | boo   | baa    |

    Returns
    -------
    summed_number: int | float
        The sum of _param_a_ and _param_b_.
    None
        None returned if values are negative.

    Raises
    ------
    ValueError
        Raises value error if params are not numbers.

    Notes
    -----
    ```python
    print(mock_function(1, 2))
    # prints 3
    ```

    Random text

    _Random table_

    | col A |: col B |
    |-------|--------|
    | boo   | baa    |

    """
    if param_a < 0 or param_b < 0:
        return
    if not isinstance(param_a, (int, float)) or not isinstance(param_b, (int, float)):
        raise ValueError('Whoops')
    return param_a + param_b


class ParentClass:
    """
    A parent class
    """
    parent_prop: str

    def __init__(self, **kwargs: dict):
        """
        Parent initialisation.

        Parameters
        ----------
        kwargs
            Keyword args.
        """
        self.parent_prop = 'bee'


class ChildClass(ParentClass):
    """
    A child class
    """

    def __init__(self,
                 param_c: float = 1.1,
                 param_d: float = 0.9,
                 **kwargs: dict):
        """
        Child initialisation.

        Parameters
        ----------
        param_c
            Yet another test param.
        param_d
            And another.
        kwargs
            Keyword args.
        """
        super().__init__(**kwargs)
        self.param_c = param_c
        self.param_d = param_d
        print(f'ignoring {kwargs}')

    @property
    def param_e(self):
        """ A property describing property param_e """
        return self.param_c + self.param_d

    def hello(self) -> str:
        """
        A random class method returning "hello"

        Returns
        -------
        str: saying_hello
            A string saying "hello"
        """
        saying_hello = 'hello'
        return saying_hello



#!python
import argparse

from yapper import handler


# prepare args
arg_parser = argparse.ArgumentParser(description='Load YAML configuration file for yapper.')
arg_parser.add_argument('--config',
                        type=str,
                        help='Relative file path to the configuration file.',
                        default=None,
                        required=False)


def cli():
    """ """
    args = arg_parser.parse_args()
    config_file = handler.load_config(args)
    yap_config = handler.process_config(config_file)
    handler.main(yap_config)


if __name__ == '__main__':
    cli()



#!python
import ast
import argparse
import logging
from pathlib import Path
import sys

import yaml

from yapper import parser


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# template configs
yap_template_config = {
    'package_root_relative_path': '.',
    'intro_template': None,
    'outro_template': None,
    'module_map': None
}


def load_config(args: argparse.Namespace) -> dict:
    logger.info('Parsing config.')
    file_name = None
    if hasattr(args, 'config'):
        file_name = args.config
    if file_name is None:
        for name in ('.yap_config.yaml', '.yap_config.yml'):
            fp = Path(Path.cwd() / name)
            if fp.exists():
                file_name = name
                break
    if file_name is None:
        raise ValueError('Yapper requires either a "--config" command-line parameter with a valid relative filepath '
                         'as an argument, else a ".yap_config.yaml" file should be placed in the current directory.')
    if Path(file_name).is_absolute():
        file_path = Path(file_name)
    else:
        file_path = Path(Path.cwd() / file_name)
    if not file_path.exists():
        raise ValueError(f'Config file path of {file_path} does not exist.')
    logger.info(f'Loading yapper config from {file_path}')
    return yaml.load(open(file_path), Loader=yaml.SafeLoader)


def process_config(yap_config: dict) -> dict:
    if 'module_map' not in yap_config:
        raise KeyError('The configuration file requires a "module_map" key.')
    if not isinstance(yap_config['module_map'], dict) or not len(yap_config['module_map'].keys()):
        raise TypeError('The "module_map" key should be a "dict" type '
                        'containing "str" keys consisting of a module names '
                        'pointing to "dict" values with "py" and "astro" keys '
                        'corresponding to input and output filepaths.')
    for module_name, module_paths in yap_config['module_map'].items():
        if not isinstance(module_paths, dict):
            raise TypeError('Module names must correspond to values of type "dict".')
        if 'py' not in module_paths.keys():
            raise KeyError('Module paths must contain a "py" key with the path to a python file for parsing.')
        if 'astro' not in module_paths.keys():
            raise KeyError('Module paths must contain an "astro" key with a destination path for the parsed content.')
        py_path = module_paths['py']
        astro_path = module_paths['astro']
        if not py_path.endswith('.py'):
            raise ValueError(f'Expecting a python input file type ending in ".py" but encountered "{py_path}"')
        if not astro_path.endswith('.astro'):
            raise ValueError(f'Expecting an astro output file type ending in ".astro" but encountered "{astro_path}"')
    # check for invalid keys
    for k in yap_config.keys():
        if k not in yap_template_config:
            raise KeyError(f'Config file key: {k} is not a valid configuration key.')
    # override defaults from config file
    merged_config = {}
    for def_key in yap_template_config.keys():
        if def_key in yap_config:
            merged_config[def_key] = yap_config[def_key]
        else:
            merged_config[def_key] = yap_template_config[def_key]

    return merged_config


def main(yap_config: dict) -> None:
    """ """
    yap_config = process_config(yap_config)
    # check and add package root path if spec'd in config
    # this should only be necessary if the script is placed somewhere other than the package root
    if 'package_root_relative_path' in yap_config:
        package_path = Path(Path.cwd() / yap_config['package_root_relative_path'])
    else:
        package_path = Path.cwd()
    logger.info(f'Adding {package_path} to Python paths')
    sys.path.append(str(package_path))
    # parse the modules
    for module_name, module_paths in yap_config['module_map'].items():
        py_path = module_paths['py']
        astro_path = module_paths['astro']
        in_path = Path(package_path / py_path)
        out_path = Path(package_path / astro_path)
        logger.info(f'Processing {in_path} to {out_path}')
        # process the module
        ast_module = ast.parse(open(in_path).read())
        astro = parser.parse(module_name=module_name,
                             ast_module=ast_module,
                             yap_config=yap_config)
        # create the path and output directories as needed
        out_file = Path(out_path)
        out_file.parent.mkdir(parents=True, exist_ok=True)
        # write!
        with open(out_file, mode='w') as out_file:
            out_file.write(astro)



import ast
import copy
from pathlib import Path

import pytest
import yaml

from yapper import handler, runner
from tests import expected

yap_clean_config = copy.deepcopy(handler.yap_template_config)


def test_load_config():
    # should raise if bad path provided
    with pytest.raises(ValueError):
        args = runner.arg_parser.parse_args(['--config', '/bad/path.boo'])
        handler.load_config(args)
    # should raise if no path provided and no valid files found in working directory
    with pytest.raises(ValueError):
        # temporarily rename .yap_config.yaml so that it is not found
        config_path = Path(Path.cwd() / './.yap_config.yaml')
        temp_path = config_path.rename('.yap_config.temp')
        args = runner.arg_parser.parse_args([])
        handler.load_config(args)
    # reset path
    temp_path.rename('.yap_config.yaml')
    # should work if relative path provided
    args = runner.arg_parser.parse_args(['--config', './.yap_config.yaml'])
    yap_config = handler.load_config(args)
    cross_check_config = yaml.load(open(config_path), Loader=yaml.SafeLoader)
    assert yap_config == cross_check_config
    # should work if absolute path provided
    args = runner.arg_parser.parse_args(['--config', str(Path('./.yap_config.yaml').absolute())])
    yap_config = handler.load_config(args)
    cross_check_config = yaml.load(open(config_path), Loader=yaml.SafeLoader)
    assert yap_config == cross_check_config


def test_process_config():
    # should raise if module_map not provided
    with pytest.raises(KeyError):
        handler.process_config({})
    # should raise if module map is not a dictionary or an empty dict
    with pytest.raises(TypeError):
        handler.process_config({'module_map': None})
    with pytest.raises(TypeError):
        handler.process_config({'module_map': {}})
    # should raise if keys are not strings or values are not dicts
    with pytest.raises(KeyError):
        handler.process_config({
            'boo': 'baa',
            0: {
                'some.module': {
                    'py': 'boo.py',
                    'astro': 'baa.astro'
                }
            }
        })
    with pytest.raises(TypeError):
        handler.process_config({
            'boo': 'baa',
            'module_map': {
                'some.module': 'not a dict'
            }
        })
    # should raise if py or astro keys are missing
    with pytest.raises(KeyError):
        handler.process_config({
            'boo': 'baa',
            'module_map': {
                'some.module': {
                    '-': 'boo.py',
                    'astro': 'baa.astro'
                }
            }
        })
    with pytest.raises(KeyError):
        handler.process_config({
            'boo': 'baa',
            'module_map': {
                'some.module': {
                    'py': 'boo.py',
                    '-': 'baa.astro'
                }
            }
        })
    # should raise if file endings are not valid
    with pytest.raises(ValueError):
        handler.process_config({
            'boo': 'baa',
            'module_map': {
                'some.module': {
                    'py': 'boo',
                    'astro': 'baa.astro'
                }
            }
        })
    with pytest.raises(ValueError):
        handler.process_config({
            'boo': 'baa',
            'module_map': {
                'some.module': {
                    'py': 'boo.py',
                    'astro': 'baa'
                }
            }
        })
    # should raise if invalid key provided
    with pytest.raises(KeyError):
        handler.process_config({
            'boo': 'baa',
            'module_map': {
                'some.module': {
                    'py': 'boo.py',
                    'astro': 'baa.astro'
                }
            }
        })
    # should replace default keys with custom keys
    # use deep copies
    config_template = {
        'package_root_relative_path': '.',
        'intro_template': None,
        'outro_template': None,
        'module_map': {
            'some.module': {
                'py': 'boo.py',
                'astro': 'baa.astro'
            }
        }
    }
    for k in ['package_root_relative_path', 'intro_template', 'outro_template']:
        yap_config = copy.deepcopy(config_template)
        yap_config[k] = 'boo'
        merged_config = handler.process_config(yap_config)
        assert merged_config[k] == 'boo'


def test_parse():
    file_path = Path('../tests/mock_file.py')
    mock_file = open(file_path)
    ast_module = ast.parse(mock_file.read())
    # using the basic config
    astro = handler.parser.parse(module_name='tests.mock_file',
                                ast_module=ast_module,
                                yap_config=yap_clean_config)
    assert astro.strip() == expected.lines_default.strip()
    # using the custom config
    args = runner.arg_parser.parse_args(['--config', './.yap_config_custom.yaml'])
    yap_config = handler.load_config(args)
    merged_config = handler.process_config(yap_config)
    astro = handler.parser.parse(module_name='tests.mock_file',
                                ast_module=ast_module,
                                yap_config=merged_config)
    assert astro.strip() == expected.lines_custom.strip()


def test_main():
    # using the default config
    args = runner.arg_parser.parse_args(['--config', './.yap_config_basic.yaml'])
    yap_config = handler.load_config(args)
    merged_config = handler.process_config(yap_config)
    handler.main(merged_config)
    # verify the output file
    with open('mock_basic_file.astro') as astro_file:
        assert astro_file.read().strip() == expected.astro_file_default.strip()
    # using the custom yap_config
    args = runner.arg_parser.parse_args(['--config', './.yap_config_custom.yaml'])
    yap_config = handler.load_config(args)
    merged_config = handler.process_config(yap_config)
    handler.main(merged_config)
    with open('mock_custom_file.astro') as astro_file:
        assert astro_file.read().strip() == expected.astro_file_custom.strip()
